package com.gic.store.service.outer;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.log.api.dto.SystemSetLogDTO;
import com.gic.log.api.service.LogApiService;
import com.gic.store.constant.StoreEnableOrDisAbleEnum;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dao.mapper.TabStoreGroupMapper;
import com.gic.store.dto.*;
import com.gic.store.entity.*;
import com.gic.store.service.*;
import com.gic.store.utils.ErrorCode;
import com.gic.store.utils.field.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/6/27
 */
@Service("storeApiService")
@SuppressWarnings("unchecked")
public class StoreApiServiceImpl implements StoreApiService {
    private static final Logger logger = LogManager.getLogger(StoreApiServiceImpl.class);
    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreBrandService storeBrandService;
    @Autowired
    private StorePhotoService storePhotoService;
    @Autowired
    private StoreBusinessTimeService storeBusinessTimeService;
    @Autowired
    private TabStoreGroupMapper tabStoreGroupMapper;
    @Autowired
    private StoreExtendService storeExtendService;
    @Autowired
    private StoreStrategyApiService storeStrategyApiService;
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StoreStrategyService storeStrategyService;
    @Autowired
    private StoreFieldSelectService storeFieldSelectService;
    @Autowired
    private LogApiService logApiService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> saveOrUpdate(StoreDTO storeDTO) {

        Integer count = storeService.countByStoreCode(storeDTO.getEnterpriseId(), storeDTO.getStoreCode(), storeDTO.getStoreId());
        if (count > 0) {
            return ServiceResponse.failure(ErrorCode.ERR_6.getCode(), "关联主键参数已经存在");
        }
        Integer completeStatus = storeService.checkCompleteStatus(storeDTO);
        storeDTO.setCompleteStatus(completeStatus);
        // 门店自身信息
        if (storeDTO.getStoreId() == null) {
            // getGroup
            String msg = getGroup(storeDTO);
            if (StringUtils.isNotBlank(msg)) {
                return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), msg);
            }
            // 启用策略
            getStatus(storeDTO);
            Integer id = storeService.save(storeDTO);
            storeDTO.setStoreId(id);
        } else {
            storeService.update(storeDTO);
        }
        // 自定义属性
        storeExtendService.saveOrUpdate(storeDTO.getEnterpriseId(), storeDTO.getStoreId(), storeDTO.getStoreExtendList());
        // 门店图片
        List<Integer> delPhotos = null;
        if (StringUtils.isNotEmpty(storeDTO.getDelPhotos())) {
            List<String> photoIds = Arrays.asList(storeDTO.getDelPhotos().split(","));
            delPhotos = photoIds.stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        storePhotoService.editStoreImage(storeDTO.getEnterpriseId(), storeDTO.getStoreId(), storeDTO.getPhotoList(), delPhotos);
        // 营业时间
        List<Integer> delBusinessTimes = null;
        if (StringUtils.isNotEmpty(storeDTO.getDelBusinessTimes())) {
            List<String> delBusinessTimeIds = Arrays.asList(storeDTO.getDelBusinessTimes().split(","));
            delBusinessTimes = delBusinessTimeIds.stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        storeBusinessTimeService.editStoreBusinessTime(storeDTO.getEnterpriseId(), storeDTO.getStoreId(), storeDTO.getBusinessTimeList(), delBusinessTimes);
        // 写日志
        SystemSetLogDTO systemSetLogDTO = new SystemSetLogDTO();
        systemSetLogDTO.setEnterpriesId(storeDTO.getEnterpriseId() + "");
        systemSetLogDTO.setLogTime(new Date());
        systemSetLogDTO.setInterfaceName("com.gic.store.service.StoreApiService.saveOrUpdate");
        systemSetLogDTO.setContent("门店信息变更");
        systemSetLogDTO.setRelationId(storeDTO.getStoreId().longValue());
        logApiService.saveSystemSetLog(systemSetLogDTO);
        return ServiceResponse.success();
    }

    /**
     * 根据策略判断启用状态
     * @param storeDTO
     */
    private void getStatus(StoreDTO storeDTO) {
        String hitStrategy = storeStrategyService.isHitStrategy(storeDTO, StoreGroupConstant.STORE_STRATEGY_TYPE);
        storeDTO.setStatus(StringUtils.isNotBlank(hitStrategy) ? Integer.valueOf(hitStrategy) : StoreEnableOrDisAbleEnum.DISABLE.getCode());
    }

    /**
     * 根据策略判断分组
     * @param storeDTO
     * @return
     */
    private String getGroup(StoreDTO storeDTO) {
        if (Integer.valueOf(1) == storeStrategyApiService.getStoreGroupStrategyWeight(storeDTO.getEnterpriseId()).getResult()) {
            // 不启用分组策略
            if (StringUtils.isBlank(storeDTO.getStoreGroupName())) {
                // 取 未分类 分组
                TabStoreGroup group = storeGroupService.selectUnGroupedStore(storeDTO.getEnterpriseId());
                storeDTO.setStoreGroupId(group.getStoreGroupId());
            } else {
                // 判断分组是否存在
                TabStoreGroup group = storeGroupService.getStoreGroupByName(storeDTO.getStoreGroupName(), storeDTO.getEnterpriseId());
                if (group != null) {
                    storeDTO.setStoreGroupId(group.getStoreGroupId());
                } else {
                    logger.info("enterpriseId:{}, groupName:{}, 不存在该分组", storeDTO.getEnterpriseId(), storeDTO.getStoreGroupName());
                    return "门店分组不存在";
                }
            }
        } else {
            // 启用分组策略
            String groupId = storeStrategyService.isHitStrategy(storeDTO, StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE);
            if (StringUtils.isNotBlank(groupId)) {
                // 命中
                storeDTO.setStoreGroupId(Integer.valueOf(groupId));
            } else {
                // 未命中
                TabStoreGroup group = storeGroupService.selectUnGroupedStore(storeDTO.getEnterpriseId());
                storeDTO.setStoreGroupId(group.getStoreGroupId());
            }
        }
        return null;
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> listStore(StoreDTO storeDTO, Integer pageNum, Integer pageSize) {
        // TODO es 暂时先查询数据库
        com.github.pagehelper.Page storeList = storeService.listStore(storeDTO, pageNum, pageSize);
        Page<StoreDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeList, StoreDTO.class);
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<StoreDTO> getStoreById(Integer enterpriseId, Integer storeId) {
        TabStore store = storeService.getById(storeId);
        if (store == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "门店不存在");
        }
        StoreDTO storeDTO = EntityUtil.changeEntityByOrika(StoreDTO.class, store);
        TabStoreGroup storeGroup = tabStoreGroupMapper.selectByPrimaryKey(storeId);
        if (storeGroup != null) {
            storeDTO.setStoreGroupName(storeGroup.getStoreGroupName());
        }
        // 门店照片
        List<TabStorePhoto> tabPhotoList = storePhotoService.listStorePhoto(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
        List<StorePhotoDTO> photoDTOList = EntityUtil.changeEntityListByOrika(StorePhotoDTO.class, tabPhotoList);
        storeDTO.setPhotoList(photoDTOList);
        // 营业时间
        List<TabStoreBusinessTime> tabBusinessTimeList = storeBusinessTimeService.listBusinessTime(store.getStoreId());
        List<StoreBusinessTimeDTO> businessTimeDTOList = EntityUtil.changeEntityListByOrika(StoreBusinessTimeDTO.class, tabBusinessTimeList);
        storeDTO.setBusinessTimeList(businessTimeDTOList);
        storeBusinessTimeService.convertBusinessTime(businessTimeDTOList);
        // 门店品牌
        List<TabStoreBrand> storeBrandList = storeBrandService.listStoreBrandByIds(store.getBrandIds());
        List<StoreBrandDTO> brandDTOList = EntityUtil.changeEntityListByOrika(StoreBrandDTO.class, storeBrandList);
        storeDTO.setBrandList(brandDTOList);

        // 自定义属性
        List<TabStoreExtend> extendList = storeExtendService.listByStoreId(enterpriseId, storeId);
        List<StoreExtendDTO> extendDTOList = storeExtendService.convertStoreExtendToDTO(extendList);
        storeDTO.setStoreExtendList(extendDTOList);

        return ServiceResponse.success(storeDTO);
    }

    @Override
    public ServiceResponse validateFieldExceptRepeat(String value, StoreFieldDTO storeFieldDTO) {
        Integer storeFieldType = storeFieldDTO.getStoreFieldType();
        String storeFieldDetail = storeFieldDTO.getStoreFieldDetail();
        FieldBase fieldBase;
        if (StoreFieldTypeEnum.TEXT.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, TextField.class);
        } else if (StoreFieldTypeEnum.RADIO.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, RadioField.class);
        } else if (StoreFieldTypeEnum.CHECK.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, CheckField.class);
        } else if (StoreFieldTypeEnum.REAL_NUMBER.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, RealNumberField.class);
        } else if (StoreFieldTypeEnum.DATE.getCode() == storeFieldType) {
            fieldBase = JSON.parseObject(storeFieldDetail, DateTypeField.class);
        } else {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "不存在该自定义属性类型");
        }
        String msg;
        if (StoreFieldTypeEnum.RADIO.getCode() == storeFieldType || StoreFieldTypeEnum.CHECK.getCode() == storeFieldType) {
            List<TabStoreFieldSelect> fieldSelectList = storeFieldSelectService.listStoreFieldSelect(storeFieldDTO.getStoreFieldId(), null, 1, Integer.MAX_VALUE);
            List<String> nameList = fieldSelectList.stream().map(TabStoreFieldSelect::getStoreFieldSelectName).collect(Collectors.toList());
            msg = fieldBase.validateSelect(value, nameList);
        } else {
            msg = fieldBase.validate(value);
        }

        if (StringUtils.isBlank(msg)) {
            return ServiceResponse.success();
        } else {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), storeFieldDTO.getStoreFieldName() + "  " + msg);
        }
    }

    @Override
    public ServiceResponse<Integer> costStoreMove(String storeIds, String toStoreId) {
        return null;
    }

    @Override
    public ServiceResponse<Page<StoreDTO>> listStoreOverFlow(Integer enterpriseId, Page<StoreDTO> page, String search) {
        com.github.pagehelper.Page<StoreDTO> p = this.storeService.listStoreOverFlow(enterpriseId, page.getCurrentPage(),
                page.getPageSize(), search);
        page = PageHelperUtils.changePageHelperToCurrentPage(p, StoreDTO.class);
        return ServiceResponse.success(page);
    }
}
